import os
import secrets
import shutil
import subprocess
from importlib import reload
from pathlib import Path

import pytest
from django.core.exceptions import ImproperlyConfigured
from django_production.__main__ import START_MARKER, do_patch


@pytest.fixture(autouse=True)
def setup_testproj(tmpdir_factory):
    """Create an empty project and set DJANGO_SETTINGS_MODULE"""
    orig_env = os.environ.get("DJANGO_SETTINGS_MODULE")
    orig_dir = os.getcwd()
    os.environ["DJANGO_SETTINGS_MODULE"] = "testproj.settings"
    tmpdir = tmpdir_factory.mktemp("test")
    os.chdir(tmpdir)
    subprocess.check_call(["django-admin", "startproject", "testproj"])
    os.chdir("testproj")
    yield
    os.chdir(orig_dir)
    shutil.rmtree(tmpdir)
    if orig_env is not None:
        os.environ["DJANGO_SETTINGS_MODULE"] = orig_env


@pytest.fixture
def django_env_prod():
    """Set expected production environment variables"""
    orig_env = os.environ.copy()
    os.environ.update(
        {
            "DJANGO_ENV": "production",
            "SECRET_KEY": secrets.token_urlsafe(60),
            "SECURE_HSTS_INCLUDE_SUBDOMAINS": "true",
        }
    )
    yield
    os.environ = orig_env


def test_missing_env_var():
    """Patching fails without DJANGO_SETTINGS_MODULE"""
    del os.environ["DJANGO_SETTINGS_MODULE"]
    with pytest.raises(ImproperlyConfigured):
        do_patch()


def test_cli():
    """CLI is installed properly"""
    subprocess.check_call(["django-production-apply"])
    assert START_MARKER in Path("testproj/urls.py").read_text()
    assert START_MARKER in Path("testproj/settings.py").read_text()


def test_idempotent():
    """Patch is only applied once"""
    settings_file = Path("testproj/settings.py")
    orig_settings = settings_file.read_text()
    do_patch()
    with pytest.raises(RuntimeError):
        do_patch()
    settings_file.write_text(orig_settings)
    with pytest.raises(RuntimeError):
        do_patch()


def test_python():
    """Python code is functional after patching"""
    do_patch()
    from testproj import urls

    reload(urls)
    assert len(urls.urlpatterns) == 2

    from testproj import settings

    reload(settings)
    assert "django_webserver" in settings.INSTALLED_APPS


def test_django_env_dev():
    """Production settings are not applied in development"""
    do_patch()
    from testproj import settings

    reload(settings)
    assert settings.DEBUG is True
    assert not hasattr(settings, "CSRF_COOKIE_SECURE")


def test_django_env_prod(django_env_prod):
    """Production settings are applied in production"""
    do_patch()
    from testproj import settings

    reload(settings)
    assert settings.DEBUG is False
    assert settings.CSRF_COOKIE_SECURE is True


def test_django_check_deploy(django_env_prod):
    """django check --deploy passes after patching"""
    do_patch()
    subprocess.check_call(["./manage.py", "check", "--deploy", "--fail-level=DEBUG"])



import os
import sys
from importlib import import_module
from pathlib import Path
from types import ModuleType

import django

START_MARKER = "\n# BEGIN: added by django-production"
END_MARKER = "# END: added by django-production\n"


def patch_settings(settings: ModuleType) -> None:
    settings_file = Path(settings.__file__)
    production_settings_file = Path(__file__).parent / "settings.py"
    if START_MARKER in settings_file.read_text():
        raise RuntimeError(
            "It looks like this settings file already contains the django-production patch."
        )
    patch = "\n".join([START_MARKER, production_settings_file.read_text(), END_MARKER])
    with settings_file.open(mode="a") as f:
        f.write(patch)


def patch_urlconf(settings: ModuleType) -> None:
    urlconf = settings.ROOT_URLCONF
    urlconf_mod = import_module(urlconf)
    urlconf_file = Path(urlconf_mod.__file__)
    if START_MARKER in urlconf_file.read_text():
        raise RuntimeError(
            "It looks like this urlconf file already contains the django-production patch."
        )
    patch_parts = [
        START_MARKER,
        "from django.urls import include",
        'urlpatterns.insert(0, path("-/", include("django_alive.urls")))',
        END_MARKER,
    ]
    with urlconf_file.open(mode="a") as f:
        f.write("\n".join(patch_parts))


def do_patch():
    # if Django project isn't installed, add the current directory to the Python path so it can be imported
    sys.path.insert(0, os.getcwd())
    django.setup()
    settings = import_module(os.environ["DJANGO_SETTINGS_MODULE"])
    patch_settings(settings)
    patch_urlconf(settings)



import os
import re

import environ

env = environ.Env()

DJANGO_ENV = env.str("DJANGO_ENV", "dev")

# Allow running webserver from manage.py
INSTALLED_APPS.append("django_webserver")

# Insert whitenoise middleware
try:
    MIDDLEWARE.insert(
        MIDDLEWARE.index("django.middleware.security.SecurityMiddleware") + 1,
        "whitenoise.middleware.WhiteNoiseMiddleware",
    )
except ValueError:
    MIDDLEWARE.insert(0, "whitenoise.middleware.WhiteNoiseMiddleware")

# skip host checking for healthcheck URLs
MIDDLEWARE.insert(0, "django_alive.middleware.healthcheck_bypass_host_check")

X_FRAME_OPTIONS = "DENY"
REFERRER_POLICY = "same-origin"

if DJANGO_ENV == "production":
    DEBUG = env.bool("DEBUG", default=False)
    SECRET_KEY = env.str("SECRET_KEY")
    if "DATABASE_URL" in os.environ:
        DATABASES = {"default": env.db("DATABASE_URL")}
    ALLOWED_HOSTS = env.list("ALLOWED_HOSTS", default=["*"])
    # Load cache from CACHE_URL or REDIS_URL
    if "CACHE_URL" in os.environ:
        CACHES = {"default": env.cache("CACHE_URL")}
    elif "REDIS_URL" in os.environ:
        CACHES = {"default": env.cache("REDIS_URL")}
    # Security
    CSRF_COOKIE_SECURE = True
    SESSION_COOKIE_SECURE = True
    SECURE_BROWSER_XSS_FILTER = True
    SECURE_CONTENT_TYPE_NOSNIFF = True

    # HTTPS only behind a proxy that terminates SSL/TLS
    SECURE_SSL_REDIRECT = True
    SECURE_REDIRECT_EXEMPT = [r"^-/"]
    SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTO", "https")
    SECURE_HSTS_SECONDS = 31536000
    SECURE_HSTS_PRELOAD = True
    # Only set this to True if you are certain that all subdomains of your domain should be served exclusively via SSL.
    SECURE_HSTS_INCLUDE_SUBDOMAINS = env.bool(
        "SECURE_HSTS_INCLUDE_SUBDOMAINS", default=False
    )

    # Staticfiles
    STATICFILES_STORAGE = "whitenoise.storage.CompressedManifestStaticFilesStorage"
    # Match filename with 12 hex digits before the extension
    WHITENOISE_IMMUTABLE_FILE_TEST = lambda path, url: re.match(  # noqa: E731
        r"^.+\.[0-9a-f]{12}\..+$", url
    )
